//-----------------------------------------------------------------------------
// TrackDemoDlg.CPP
//
// The Logic IO TrackDemo application, consists of a PC application and a RTCU application.
// The Demo demonstrates a simple Vehicle Tracking application, where a RTCU Unit sends 
// regular positions to a PC application, using GPRS communication and the Logic IO RTCU 
// Gateway. The RTCU application can receive a number of commands such as period between 
// position updates, nodeid of the PC application it should send updates to etc. The PC 
// application is able to receive position updates from the RTCU, as well as issue the 
// different commands that the unit can understand. The PC application ses the VSMSGW DLL, 
// that is part of the Logic IO GPRS Deployment Package, available at www.logicio.com. The 
// VSMSGW DLL is used as the layer between the PC application and the RTCU Gateway. This 
// simplifies greatly the PC application, as it has no need for handling low-level 
// communication with the RTCU Gateway itself. Using both the RTCU application and the 
// PC application, it is easy to extend the functionality into a full-blown tracking application.
//

#include "stdafx.h"
#include "TrackDemo.h"
#include "TrackDemoDlg.h"
#include "SetupDlg.h"
#include "..\vsmsgw\VSMSGW.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


static UINT BASED_CODE indicators[] =
{
  ID_INDICATOR_STATUS,
  ID_INDICATOR_TIME
};

// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
  CAboutDlg();

// Dialog Data
  enum { IDD = IDD_ABOUTBOX };

// Implementation
protected:
  virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
  DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
END_MESSAGE_MAP()


// CTrackDemoDlg dialog

//=================================================================================================
//
// Utility functions to handle conversion from Unicode Windows application
// to ANSI dll libraries and RTCU units.
//

void TextToDLL(const CString &text, char *dlltext, int maxsize)
{
#ifdef _UNICODE
  //First call the function to determine how much space we need to allocate
  int nLength = text.GetLength();
  int dllLength = WideCharToMultiByte(CP_UTF8, 0, text, nLength, NULL, 0, NULL, NULL);
  if (nLength >= maxsize) nLength = maxsize - 1;

  //Now recall with the buffer to get the converted text
	int nCharsWritten = WideCharToMultiByte(CP_UTF8, 0, text, nLength, dlltext, dllLength, NULL, NULL);
  dlltext[nCharsWritten] = '\0';
#else
  strcpy_s(dlltext, maxsize, text);
#endif
}

CString TextFromDLL(const char *dlltext, int nLength)
{
#ifdef _UNICODE
  //First call the function to determine how much space we need to allocate
	int nWideLength = MultiByteToWideChar(CP_UTF8, 0, dlltext, nLength, NULL, 0);

  //If the calculated length is zero, then ensure we have at least room for a NULL terminator
  if (nWideLength == 0) nWideLength = 1;

  //Now recall with the buffer to get the converted text
  CString sWideString;
  wchar_t* pszWText = sWideString.GetBuffer(nWideLength + 1); //include an extra byte because we may be null terminating the string ourselves
	int nCharsWritten = MultiByteToWideChar(CP_UTF8, 0, dlltext, nLength, pszWText, nWideLength);
	
	//Ensure we NULL terminate the text if MultiByteToWideChar doesn't do it for us
	if (nLength != -1)
	{
	  AFXASSUME(nCharsWritten <= nWideLength);
	  pszWText[nCharsWritten] = '\0';
	}
	
	sWideString.ReleaseBuffer();

	return sWideString;
#else
  return CString(dlltext);
#endif
}
//=================================================================================================


//=================================================================================================
//-----------------------------------------------------------------------------
// This function gets called every time a PDU SMS message is received from an
// RTCU unit.
//-----------------------------------------------------------------------------
static int _cdecl cbPDU(int HisNodeID, unsigned char *data, int length, void* arg)
{
  // Var
  static int count=1;
  CString   str;

  // Extract command code from message
  unsigned char command=data[0];
  // Point to data after command code
  data++;

  // Look at the command code:
  switch (command) {
  case 0x10: {
    // Define a structure that matches the data sent from the RTCU Unit
    // Align the following union on 1 byte boundary
    #pragma pack (push, before)
    #pragma pack(1)
    typedef struct {
      unsigned char Mode;      // Mode of GPS, 0=info not available, 1=No fix, 2=2D, 3=3D fix
      long          lat;       // latitude
      long          lon;       // Longitude
      long          date;      // Date (UTC)
      long          time;      // Time (UTC)
      short         speed;     // Speed in KM/H (resolution 0.1 KM/H)
      short         direction; // Direction in degrees (resolution 0.1 deg)
    } tdefPositionData;
    #pragma pack(pop, before)

    // Update Unit ID
    str.Format(_T("%i"), HisNodeID);
    ((CTrackDemoDlg*)arg)->m_ctrfromnodeid.SetWindowText(str);

    // Update Mode 
    switch (((tdefPositionData*)data)->Mode) {
      case 0: ((CTrackDemoDlg*)arg)->m_ctrmode.SetWindowText(_T("No info")); break;
      case 1: ((CTrackDemoDlg*)arg)->m_ctrmode.SetWindowText(_T("No Fix")); break;
      case 2: ((CTrackDemoDlg*)arg)->m_ctrmode.SetWindowText(_T("2D Fix")); break;
      case 3: ((CTrackDemoDlg*)arg)->m_ctrmode.SetWindowText(_T("3D Fix")); break;
      case 4: ((CTrackDemoDlg*)arg)->m_ctrmode.SetWindowText(_T("3D Fix (SBAS)")); break;
      default: ((CTrackDemoDlg*)arg)->m_ctrmode.SetWindowText(_T("Unknown")); break;
    }

    // Update lattitude 
    str.Format(_T("%i"), ((tdefPositionData*)data)->lat);
    ((CTrackDemoDlg*)arg)->m_ctrlat.SetWindowText(str);

    // Update longitude
    str.Format(_T("%i"), ((tdefPositionData*)data)->lon);
    ((CTrackDemoDlg*)arg)->m_ctrlon.SetWindowText(str);

    // Update speed
    str.Format(_T("%i.%i"), ((tdefPositionData*)data)->speed/10, ((tdefPositionData*)data)->speed%10);
    ((CTrackDemoDlg*)arg)->m_ctrspeed.SetWindowText(str);

    // Update direction
    str.Format(_T("%i.%i"), ((tdefPositionData*)data)->direction/10, ((tdefPositionData*)data)->direction%10);
    ((CTrackDemoDlg*)arg)->m_ctrdirection.SetWindowText(str);

    // Update time
    str.Format(_T("%i-%02i-%02i %02i:%02i:%02i"), ((tdefPositionData*)data)->date >> 16, (((tdefPositionData*)data)->date & 0x0000FF00) >> 8, ((tdefPositionData*)data)->date & 0x000000FF,
                                                  ((tdefPositionData*)data)->time >> 16, (((tdefPositionData*)data)->time & 0x0000FF00) >> 8, ((tdefPositionData*)data)->time & 0x000000FF);
    ((CTrackDemoDlg*)arg)->m_ctrtime.SetWindowText(str);
    }
    break;
  }

  // Update counter on screen
  CString s;
  s.Format(_T("%i"), count++);
  ((CTrackDemoDlg*)arg)->m_ctrupdates.SetWindowText(s);
  return 0;
}
//=================================================================================================



CTrackDemoDlg::CTrackDemoDlg(CWnd* pParent /*=NULL*/)
  : CDialog(CTrackDemoDlg::IDD, pParent)
  , m_updatefreq(0)
  , m_nodeid(0)
{
  m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CTrackDemoDlg::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_FROMNODEID, m_ctrfromnodeid);
  DDX_Control(pDX, IDC_SETNODEID, m_ctrsetnodeid);
  DDX_Control(pDX, IDC_SETFREQ, m_ctrsetfreq);
  DDX_Control(pDX, IDC_REQUESTPOS, m_ctrreq);
  DDX_Control(pDX, IDC_LAT2, m_ctrmode);
  DDX_Control(pDX, IDC_TIME, m_ctrtime);
  DDX_Control(pDX, IDC_SPEED, m_ctrspeed);
  DDX_Control(pDX, IDC_LON, m_ctrlon);
  DDX_Control(pDX, IDC_LAT, m_ctrlat);
  DDX_Control(pDX, IDC_DIRECTION, m_ctrdirection);
  DDX_Control(pDX, IDC_UPDATES, m_ctrupdates);
  DDX_Text(pDX, IDC_UPDATEFREQ, m_updatefreq);
  DDV_MinMaxInt(pDX, m_updatefreq, 0, 32767);
  DDX_Text(pDX, IDC_NODEID, m_nodeid);
}

BEGIN_MESSAGE_MAP(CTrackDemoDlg, CDialog)
  ON_WM_SYSCOMMAND()
  ON_WM_PAINT()
  ON_WM_QUERYDRAGICON()
  ON_WM_TIMER()
  ON_BN_CLICKED(IDC_SETUP, OnSetup)
  ON_WM_CANCELMODE()
  ON_BN_CLICKED(IDC_SETNODEID, OnSetnodeid)
  ON_BN_CLICKED(IDC_SETFREQ, OnSetfreq)
  ON_BN_CLICKED(IDC_REQUESTPOS, OnRequestpos)
END_MESSAGE_MAP()


// CTrackDemoDlg message handlers

BOOL CTrackDemoDlg::OnInitDialog()
{
  // Var
  CRect rect;
  int   rc;
  char  gw_addr[127], gw_key[9];
  unsigned char CryptKey[16] = {0};

  // Default
  CDialog::OnInitDialog();

  // Add "About..." menu item to system menu.

  // IDM_ABOUTBOX must be in the system command range.
  ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
  ASSERT(IDM_ABOUTBOX < 0xF000);

  CMenu* pSysMenu = GetSystemMenu(FALSE);
  if (pSysMenu != NULL) {
    CString strAboutMenu;
    strAboutMenu.LoadString(IDS_ABOUTBOX);
    if (!strAboutMenu.IsEmpty()) {
      pSysMenu->AppendMenu(MF_SEPARATOR);
      pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
    }
  }

  // Set the icon for this dialog.  The framework does this automatically
  //  when the application's main window is not a dialog
  SetIcon(m_hIcon, TRUE);			// Set big icon
  SetIcon(m_hIcon, FALSE);		// Set small icon

  // Initialize status bar
  m_bar.Create(this);                 //We create the status bar
  m_bar.SetIndicators(indicators, 2); //Set the number of panes 
  GetClientRect(&rect);
  m_bar.SetPaneInfo(0, ID_INDICATOR_STATUS, SBPS_NORMAL, rect.Width() - 70);      
  m_bar.SetPaneInfo(1, ID_INDICATOR_TIME,   SBPS_STRETCH ,0);
  RepositionBars(AFX_IDW_CONTROLBAR_FIRST, AFX_IDW_CONTROLBAR_LAST, ID_INDICATOR_TIME);

  m_bar.SetPaneText(0, _T(""));
  m_bar.SetPaneText(1, _T(""));

  // Initialize Gateway connection
  TextToDLL(AfxGetApp()->GetProfileString(_T("Options"), _T("GW IP"), _T("gw.rtcu.dk")), gw_addr, 127);
  TextToDLL(AfxGetApp()->GetProfileString(_T("Options"), _T("GW Key"), _T("AABBCCDD")), gw_key, 9);
  rc = vsmsgwInitAdv(
                     AfxGetApp()->GetProfileInt(_T("Options"), _T("My nodeid"), 2000),
                     gw_addr,
                     AfxGetApp()->GetProfileInt(_T("Options"), _T("GW Port"), 5001),
                     gw_key,
                     CryptKey
                    );
  vsmsgwSetSMSPDUCB(cbPDU, this);
  SetTimer(100, 500, NULL);   // Connection status update timer

  // Initialize dialog data
  m_nodeid     = AfxGetApp()->GetProfileInt(_T("Options"), _T("My nodeid"), 2000);
  m_updatefreq = AfxGetApp()->GetProfileInt(_T("Options"), _T("Updatefreq"), 0);
  UpdateData(FALSE);

  return TRUE;  // return TRUE  unless you set the focus to a control
}

void CTrackDemoDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
  if ((nID & 0xFFF0) == IDM_ABOUTBOX) {
    CAboutDlg dlgAbout;
    dlgAbout.DoModal();
  } else {
    CDialog::OnSysCommand(nID, lParam);
  }
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CTrackDemoDlg::OnPaint()
{
  if (IsIconic()) {
    CPaintDC dc(this); // device context for painting

    SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

    // Center icon in client rectangle
    int cxIcon = GetSystemMetrics(SM_CXICON);
    int cyIcon = GetSystemMetrics(SM_CYICON);
    CRect rect;
    GetClientRect(&rect);
    int x = (rect.Width() - cxIcon + 1) / 2;
    int y = (rect.Height() - cyIcon + 1) / 2;

    // Draw the icon
    dc.DrawIcon(x, y, m_hIcon);
  } else {
    CDialog::OnPaint();
  }
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CTrackDemoDlg::OnQueryDragIcon()
{
  return static_cast<HCURSOR>(m_hIcon);
}

void CTrackDemoDlg::OnSetup() 
{
  // Var
  CSetupDlg dlg;

  // Setup dialog
  dlg.m_gwport    = AfxGetApp()->GetProfileInt(_T("Options"), _T("GW Port"), 5001);
  dlg.m_gwip      = AfxGetApp()->GetProfileString(_T("Options"), _T("GW IP"), _T("gw.rtcu.dk"));
  dlg.m_gwkey     = AfxGetApp()->GetProfileString(_T("Options"), _T("GW Key"), _T("AABBCCDD"));
  dlg.m_mynodeid  = AfxGetApp()->GetProfileInt(_T("Options"), _T("My nodeid"), 2000);
  dlg.m_hisnodeid = AfxGetApp()->GetProfileInt(_T("Options"), _T("His nodeid"), 2001);

  // Show dialog
  if (dlg.DoModal() == IDOK) {
    // Save new configuration
    AfxGetApp()->WriteProfileInt(_T("Options"), _T("GW Port"), dlg.m_gwport);
    AfxGetApp()->WriteProfileString(_T("Options"), _T("GW IP"), dlg.m_gwip);
    AfxGetApp()->WriteProfileString(_T("Options"), _T("GW Key"), dlg.m_gwkey);
    AfxGetApp()->WriteProfileInt(_T("Options"), _T("My nodeid"), dlg.m_mynodeid);
    AfxGetApp()->WriteProfileInt(_T("Options"), _T("His nodeid"), dlg.m_hisnodeid);

    // Update dialog
    UpdateData(TRUE);
    m_nodeid = dlg.m_mynodeid;
    UpdateData(FALSE);

    // Show message
    AfxMessageBox(_T("The program must be restarted in order to use any new settings !"));
  }
}

void CTrackDemoDlg::OnCancel() 
{
  if (AfxMessageBox(_T("Are you sure you want to exit the program ?"), MB_ICONWARNING|MB_YESNO|MB_DEFBUTTON2) == IDYES) 
  	CDialog::OnCancel();
}

void CTrackDemoDlg::OnOK() 
{
  if (AfxMessageBox(_T("Are you sure you want to exit the program ?"), MB_ICONWARNING|MB_YESNO|MB_DEFBUTTON2) == IDYES) 
  	CDialog::OnOK();
}

void CTrackDemoDlg::OnTimer(UINT nIDEvent) 
{
  static bool old_Connected = true; // The first time we arrive in this function, we are not connected anyway, so this forces an update of the display...
  bool connected;

  if (nIDEvent == 100) {
    connected = vsmsgwIsConnected();
    if (connected != old_Connected) {
      // Connection status is changed
      // Write status text
      m_bar.SetPaneText(0, connected ? _T("Connected to RTCU Gateway") : _T("Connecting to RTCU Gateway...."));
      // Set timestamp
      if (connected) m_time = CTime::GetCurrentTime();
      else m_bar.SetPaneText(1, _T(""));
    }
    old_Connected = connected;

    // Update time
    if (connected) {
      CTimeSpan t1 = CTime::GetCurrentTime() - m_time;
      m_bar.SetPaneText(1, t1.Format(_T("%H:%M:%S")));
    }
  }

  CDialog::OnTimer(nIDEvent);
}

void CTrackDemoDlg::OnCancelMode() 
{
  CDialog::OnCancelMode();
}

//=================================================================================================
// This will set the time in seconds between reports sent, 0 disables sending
void CTrackDemoDlg::OnSetfreq() 
{
// Align the following union on 1 byte boundary
#pragma pack (push, before)
#pragma pack(1)
  struct {
    unsigned char Command;
    unsigned int  Period;
  } req;
#pragma pack(pop, before)
  int rc, rrc;

  UpdateData(true);

  if (0 > m_updatefreq || 32767 < m_updatefreq )
    return; // Warning popup comes from MFC

  AfxGetApp()->WriteProfileInt(_T("Options"), _T("Updatefreq"), m_updatefreq);
  
  req.Command=0x01;
  req.Period=m_updatefreq;
  
  rrc = vsmsgwSendPDU(AfxGetApp()->GetProfileInt(_T("Options"), _T("His nodeid"), 2001), (unsigned char *)&req, sizeof(req), &rc);
  if (rrc)
    MessageBox(_T("Unit not found"), _T("Set update frequency"), MB_OK|MB_ICONERROR);
  else if (rc)
    MessageBox(_T("Command rejected by unit"), _T("Set update frequency"), MB_OK|MB_ICONERROR);
}

// This will initiate a transmission of the position telegram (see above)
void CTrackDemoDlg::OnRequestpos() 
{
// Align the following union on 1 byte boundary
#pragma pack (push, before)
#pragma pack(1)
  struct {
    unsigned char Command;
  } req;
#pragma pack(pop, before)
  int rc, rrc;

  req.Command=0x02;
  
  rrc = vsmsgwSendPDU(AfxGetApp()->GetProfileInt(_T("Options"), _T("His nodeid"), 2001), (unsigned char *)&req, sizeof(req), &rc);
  if (rrc)
    MessageBox(_T("Unit not found"), _T("Set update frequency"), MB_OK|MB_ICONERROR);
  else if (rc)
    MessageBox(_T("Command rejected by unit"), _T("Request position"), MB_OK|MB_ICONERROR);
}

// This will set the nodeid for the PC application (the Host), this is where the position commands are sent to
void CTrackDemoDlg::OnSetnodeid() 
{
// Align the following union on 1 byte boundary
#pragma pack (push, before)
#pragma pack(1)
  struct {
    unsigned char Command;
    int NodeID;
  } req;
#pragma pack(pop, before)
  int rc, rrc;
  
  req.Command=0x03;
  req.NodeID=AfxGetApp()->GetProfileInt(_T("Options"), _T("My nodeid"), 2000);

  rrc = vsmsgwSendPDU(AfxGetApp()->GetProfileInt(_T("Options"), _T("His nodeid"), 2001), (unsigned char *)&req, sizeof(req), &rc);
  if (rrc)
    MessageBox(_T("Unit not found"), _T("Set update frequency"), MB_OK|MB_ICONERROR);
  else if (rc)
    MessageBox(_T("Command rejected by unit"), _T("Set host Node ID"), MB_OK|MB_ICONERROR);
}
//=================================================================================================

